const chalk = require('chalk')
const fs = require('fs-extra')

const build = require('./build')
const docker = require('./utils/docker')
const output = require('./utils/output')
const { getConfig } = require('./utils/config')
const { applyDeployment } = require('./utils/kubernetes')

module.exports = async function deploy() {
  const config = await getConfig()
  const pathText = output.param(config.pathDir)
  const clusterName = output.param(config.cluster)
  const rbacUser = output.param(config.user)
  const { name, tag } = config

  const stopDeployTimer = output.time()

  output.info(`Deploying ${pathText} on ${clusterName} as ${rbacUser}\n`)

  if (config.shouldBuild) {
    try {
      const { err: buildErr } = await build(config)

      if (buildErr) {
        return
      }
    } catch (err) {
      return
    }

    const doneWithPush = output.wait('Pushing docker image…')
    const stopPushTimer = output.time()

    try {
      await docker.push(config)

      doneWithPush()
      output.success(`Push complete ${stopPushTimer()}`)
    } catch (err) {
      doneWithPush()
      output.error('Push failed')
      console.error(err.message || err)
      return
    }
  }

  const doneWithRollout = output.wait('Applying rolling update…')
  const stopRolloutTimer = output.time()

  try {
    await applyDeployment(config)

    doneWithRollout()
    output.success(`Rollout complete ${stopRolloutTimer()}`)
  } catch (err) {
    doneWithRollout()
    output.error('Rollout failed')
    console.error(err.message || err)
    return
  }

  // STYLE: add some visual padding
  console.log('')

  const url =
    config.readyURL || config.ready_url || `http://${name}-${tag}.cluster.local`

  output.info(`Ready at ${chalk.bold(url)} ${stopDeployTimer()}\n`)

  output.info(`Monitor rollout with: ${output.cmd(`watch funk list --pods`)}`)
}
