const fs = require('fs-extra')
const exec = require('execa')
const spawn = require('cross-spawn')
const tmp = require('tmp-promise')
const output = require('./output')

module.exports = {
  // Pods
  listPods,
  sshIntoPod,
  // Services
  listServices,
  // Clusters
  getClusterInfo,
  // Deployments
  applyDeployment: kubectlDeploymentFactory('apply'),
  deleteDeployment: kubectlDeploymentFactory('delete'),
  // Definitions
  getDefinitionOptions
}

async function sshIntoPod(config = {}) {
  const { podName } = config

  try {
    output.success('Connected to ' + output.param(podName))
    const result = spawn.sync(
      'kubectl',
      ['exec', '-it', podName, '--', '/bin/sh'],
      {
        stdio: 'inherit'
      }
    )

    // TODO:
    // process stdout with a stream to
    // - look for SSH errors
    // - check for bash

    output.info('Closed the connection')
  } catch (err) {
    output.error('There was an error with your SSH connection')
    console.log(err)
  }
}

async function getClusterInfo() {
  const { stdout } = await exec('kubectl', ['config', 'view', '-o', 'json'])
  const result = JSON.parse(stdout)

  return {
    cluster: result['current-context'],
    // TODO: match local username to cluster info or find funkube api server
    user: 'admin'
  }
}

async function listPods(config = {}) {
  const { stdout, stderr } = await exec('kubectl', [
    'get',
    'pods',
    '-l',
    `app=${config.name}`
  ])

  return {
    stdout,
    stderr
  }
}

async function listServices(config = {}) {
  const { stdout, stderr } = await exec('kubectl', [
    'get',
    'services',
    '-l',
    `appService=${config.name}`
  ])

  return {
    stdout,
    stderr
  }
}

function kubectlDeploymentFactory(action) {
  return async config => {
    const { name, tag, registry } = getDefinitionOptions(config)
    const image = `${registry}/${name}:${tag}`
    const deploymentCmd = `deployment/${name}-deployment`
    const imageCmd = `${name}=${image}`
    const functionDefinition = getFunctionDefinition({ ...config, image })

    const tmpFile = await tmp.file({
      mode: 0644,
      kpostfix: `${name}-${tag}`,
      postfix: '.yaml'
    })

    try {
      await fs.writeFile(tmpFile.path, functionDefinition)

      const { stdout, stderr } = await exec('kubectl', [
        action,
        '-f',
        tmpFile.path
      ])

      // Delete the temporary file after its been applied
      tmpFile.cleanup()

      // TODO:
      // Check for github repo to
      // - see if we should commit a change
      // - move the file into a local folder (e.g. `.funk/`)

      return {
        stdout,
        err: stderr
      }
    } catch (err) {
      return {
        err
      }
    }
  }
}

function getFunctionDefinition(config) {
  return [getServiceDefinition(config), getDeploymentDefinition(config)].join(
    '\n---\n'
  )
}

function getDeploymentDefinition(config) {
  const { name, image, ports, replicas } = getDefinitionOptions(config)

  return `
apiVersion: apps/v1
kind: Deployment
metadata:
  name: ${name}-deployment
  labels:
    app: ${name}
spec:
  replicas: ${replicas}
  selector:
    matchLabels:
      app: ${name}
  template:
    metadata:
      labels:
        app: ${name}
    spec:
      containers:
      - name: ${name}
        image: ${image}
        ports:
        ${ports.map(port => `- containerPort: ${port}\n`)}`.trim()
}

function getServiceDefinition(config) {
  const { name, ports } = getDefinitionOptions(config)
  const portIndent = `\n    ` // New line + 4 spaces

  const servicePorts = ports.map((targetPort, index) => {
    if (index === 0) {
      return {
        port: 80,
        targetPort
      }
    }

    return {
      targetPort,
      port: targetPort
    }
  })

  return `
apiVersion: v1
kind: Service
metadata:
  name: ${name}
  labels:
    appService: ${name}
spec:
  type: LoadBalancer
  selector:
    app: ${name}
  ports:
  ${servicePorts.map(
    ({ port, targetPort }) =>
      `- port: ${port}${portIndent}targetPort: ${targetPort}\n`
  )}`.trim()
}

const defaultOptions = {
  registry: 'docker.io',
  port: 80,
  replicas: 1
}

function getDefinitionOptions(config) {
  const { name, type, options } = config
  const definitionOptions = {
    ...defaultOptions,
    ...options
  }

  if (!definitionOptions.ports) {
    definitionOptions.ports = [definitionOptions.port]
  }

  return {
    name,
    type,
    ...config,
    ...definitionOptions
  }
}
